# frozen_string_literal: true

require 'thor'

module Yadecli
  module Cli

    # composer
    class Composer < Thor

      # setup command
      desc 'setup', 'Setup yade composer on this host'
      def setup
        CliUtil.print_header('YadeCli Composer Setup',
                             ['Answer the following questions to setup Composer on this host:', ''])

        outcome = Yadecli::Command::Composer::ComposerSetupCommand.run

        CliUtil.print_command_outcome(outcome)
      end

      # list command
      desc 'list', 'List the available Yade composer projects'
      def list
        CliUtil.print_header('YadeCli Composer List',
                             ['This are the available Yade composer projects you can install', ''])

        outcome = Yadecli::Command::Composer::ComposerListCommand.run

        CliUtil.print_command_outcome(outcome)

        puts ''
        puts 'You can install any of the listed project with the following command:'
        puts ''
        puts '  bin/yadecli composer install <name> <branch> [--pull] [--setup] [--clean]'.colorize(:mode => :bold)
        puts ''
      end

      # install command
      desc 'install', 'Install a Yade composer project'
      method_option :clean, default: false, aliases: '-c', desc: 'Clean install, will delete a existing composer project'
      method_option :yes, default: false, aliases: '-y', desc: 'Will answer yes to all questions, This will run a git pull on installed projects without asking'
      method_option :pull, default: false, aliases: '-p', desc: 'Will pull the container belonging to the project'
      def install(project_name, branch_name = nil)
        CliUtil.print_header('YadeCli Composer Install',
                             ["Going to install Yade composer project #{project_name}", ''])

        outcome = Yadecli::Command::Composer::ComposerInstallCommand.run(project_name: project_name,
                                                                         branch_name: branch_name,
                                                                         options: options)
        CliUtil.print_command_outcome(outcome)
      end

      # switch branch command
      desc 'switch', 'Switch the branch of a installed Composer project'
      method_option :pull, default: false, aliases: '-p', desc: 'Will run a git pull after switching the branch'
      def switch(project_name, target_branch)
        CliUtil.print_header('Composer Switch',
                             ["Going to switch branch of composer project #{project_name} to #{target_branch}", ''])

        outcome = Yadecli::Command::Composer::ComposerSwitchCommand.run(project_name: project_name,
                                                                        target_branch: target_branch,
                                                                        options: options)
        CliUtil.print_command_outcome(outcome)
      end

      # uninstall command
      desc 'uninstall', 'Uninstall a Yade composer project'
      method_option :yes, default: false, aliases: '-y', desc: 'Will answer yes to all questions, This will uninstall the project without asking'
      def uninstall(project_name)
        CliUtil.print_header('YadeCli Composer Uninstall',
                             ["Going to uninstall composer project #{project_name}", ''])

        outcome = Yadecli::Command::Composer::ComposerUninstallCommand.run(project_name: project_name,
                                                                           options: options)
        CliUtil.print_command_outcome(outcome)
      end

      # start command
      desc 'start', 'Start a Yade composer service'
      def start(project_name, service_name)
        CliUtil.print_header('YadeCli Composer Start',
                             ['Going to start composer service:',
                              " ↳ Composer Project : #{project_name}",
                              " ↳ Service Name     : #{service_name}"])

        outcome = Yadecli::Command::Composer::ComposerStartCommand.run(project_name: project_name,
                                                                       service_name: service_name,
                                                                       options: options)
        CliUtil.print_command_outcome(outcome)
      end

      # stop command
      desc 'stop', 'Stop a Yade Composer service'
      method_option :rm, default: false, aliases: '-r', desc: 'Remove the docker container after stopping'
      method_option :down, default: false, aliases: '-r', desc: 'Run docker-compose down -v'
      def stop(project_name, service_name)
        CliUtil.print_header('Composer Service Stop',['Going to stop composer service:',
                                                      " ↳ Composer Project : #{project_name}",
                                                      " ↳ Service Name     : #{service_name}"])

        outcome = Yadecli::Command::Composer::ComposerStopCommand.run(project_name: project_name,
                                                                      service_name: service_name,
                                                                      options: options)

        CliUtil.print_command_outcome(outcome)
      end

      # log container output of service
      desc 'log', 'Stop a Yade Composer service'
      method_option :follow, default: false, aliases: '-f', desc: 'Log the docker containers output of the service'
      def log(project_name, service_name)
        CliUtil.print_header('Composer Service Log',["Going to show logs for composer service #{service_name}.",
                                                     'Hit ctrl + c to stop when you use the -f option'])

        outcome = Yadecli::Command::Composer::ComposerLogCommand.run(project_name: project_name,
                                                                     service_name: service_name,
                                                                     options: options)

        CliUtil.print_command_outcome(outcome)
      end
    end
  end
end