# frozen_string_literal: true

require 'mutations'
require 'uri'

module Yadecli
  module Command
    module Composer
      class ComposerInstallCommand < Mutations::Command

        required do
          string :project_name, empty: false
        end

        optional do
          string :branch_name, empty: false
          hash :options do
            string :* # Allows any key to pass through. Useful for dynamic key / value objects
          end
        end

        #
        def execute
          composer_project_client = Yade::Composer::Rest::Client::ComposerProjectClient.new
          composer_service_client = Yade::Composer::Rest::Client::ComposerServiceClient.new

          composer_project = composer_project_client.get_by_name(self.project_name)

          # delete already installed when option --clean given
          FileUtils.rm_rf(composer_project.install_dir) if options[:clean]

          # check preconditions
          check_install_preconditions(options)

          # check already installed
          already_installed = composer_project.installed?

          # ask for branch if not given from commandline
          self.branch_name = UserInput.select_branch(composer_project.git_repo_name) if self.branch_name == nil && !already_installed && !options[:yes]

          if already_installed
            puts "Composer project #{project_name} already installed.".colorize(color: :blue, mode: :bold)
            puts '  ↳ Doing a git pull.'

            pull_repository(composer_project, options)
          else
            puts "Composer project #{project_name} will be installed.".colorize(color: :blue, mode: :bold)
            puts '  ↳ Doing a git clone.'

            clone_repository(composer_project, branch_name)
          end

          puts ''
          if already_installed
            puts "  Successfully updated '#{composer_project.name}' at #{composer_project.install_dir}."
          else
            puts "  Successfully installed '#{composer_project.name}' to #{composer_project.install_dir}."
          end

          composer_services = composer_service_client.get_by_composer_project_id(composer_project.id)
          service_names = composer_services.collect(&:name).join(' | ')

          pull_container(composer_project) if options[:pull]

          """
  Use the following commands to manage the service(s)
    Start : bin/yadecli composer start #{composer_project.name} #{service_names}
    Stop  : bin/yadecli composer stop  #{composer_project.name} #{service_names}
          """
        end

        private

        # check install preconditions
        def check_install_preconditions(options)
          cmd = TTY::Command.new(output: Yadecli.LOGGER)

          cmd.run('test -x /usr/bin/docker') if SystemUtil.os == 'linux'
          cmd.run('test -x /usr/local/bin/docker') if SystemUtil.os == 'macosx'

          cmd.run('test -x /usr/local/bin/docker-compose')
        end

        # clone repository
        def clone_repository(composer_project, branch_name)
          username = Yade::Common::Config::ComposerConfig[:git_username]
          password = Yade::Common::Config::ComposerConfig[:git_password]

          git_url = URI.parse(composer_project.gitUrl)

          scheme = git_url.scheme
          host = git_url.host
          path = git_url.path

          url_with_auth = "#{scheme}://#{username}:#{password}@#{host}#{path}"

          cmd = TTY::Command.new(output: Yadecli.LOGGER)
          cmdline = "git clone --depth=1 "
          cmdline += "-b #{branch_name} " if branch_name != nil
          cmdline += "#{url_with_auth} #{composer_project.install_dir}"

          cmd.run(cmdline, only_output_on_error: true)
        end

        # pull repository
        def pull_repository(composer_project, options)
          if options[:yes]
            answer = true
          else
            puts ''
            answer = TTY::Prompt.new.yes?('Do you want to pull the changes for the repository?')
          end

          if answer
            cmd = TTY::Command.new(output: Yadecli.LOGGER)
            cmd.run('git pull', only_output_on_error: true, chdir: composer_project.install_dir)
          else
            puts 'There is nothing more I can do for you. Bye.'
            exit(1)
          end
        end

        # pull container
        def pull_container(composer_project)
          # make shure we are logged in to the docker registry
          docker_login

          cmd = TTY::Command.new(output: Yadecli.LOGGER)

          puts ''
          puts 'Pulling docker images:'.colorize(color: :blue)

          doco = YAML.load_file("#{composer_project.install_dir}/docker-compose.yml")

          has_failed_pulls = false
          doco['services'].each do |k, v|
            cmdline = "docker-compose pull #{k}"

            print " ↳  #{cmdline}... "
            begin
              cmd.run(cmdline, chdir: composer_project.install_dir)

              puts 'done'.colorize(:green)
            rescue TTY::Command::ExitError => e
              puts 'failed'.colorize(:red)
              has_failed_pulls = true
            end
          end

          puts ''
          puts 'Successfully pulled all docker containers.'.colorize(:green) unless has_failed_pulls
          puts "Failed to pull for at least on container. See log at #{Dir.pwd}/composer.log".colorize(:red) if has_failed_pulls
        end

        # docker login
        def docker_login
          cmd = TTY::Command.new(output: Yadecli.LOGGER)

          cmdline = "docker login #{Yade::Common::Config::ComposerConfig[:docker_registry_url]} -u #{Yade::Common::Config::ComposerConfig[:docker_registry_username]} -p #{Yade::Common::Config::ComposerConfig[:docker_registry_password]}"

          cmd.run(cmdline, only_output_on_error: true)
        end
      end
    end
  end
end