# frozen_string_literal: true

require 'mutations'
require 'dotenv'

module Yadecli
  module Command
    module Composer
      class ComposerStartCommand < Mutations::Command

        required do
          string :project_name, empty: false
          string :service_name, empty: false
        end

        optional do
          hash :options do
            string :* # Allows any key to pass through. Useful for dynamic key / value objects
          end
        end

        #
        def execute
          composer_project_client = Yade::Composer::Rest::Client::ComposerProjectClient.new
          composer_service_client = Yade::Composer::Rest::Client::ComposerServiceClient.new
          composer_container_client = Yade::Composer::Rest::Client::ComposerContainerClient.new

          begin
            composer_project = composer_project_client.get_by_name(project_name)
            composer_services = composer_service_client.get_by_composer_project_id(composer_project.id)

            # load project dotenv
            dotenv_file = "#{composer_project.install_dir}/.env"
            Dotenv.load(dotenv_file)

            # check required envs are set fom .env file or system environment variables
            composer_service = composer_services.select { |s| s.name == service_name }.first

            composer_service.requiredEnvs&.split(',').each do |re|
              if ENV[re] == nil
                puts ''
                puts "Unable to start service. Required environment variable #{re} is not set.".colorize(:red)
                puts "Either specify the variable and a value in #{dotenv_file} or as a system environment variable an try to run again.".colorize(:red)

                exit 1
              end
            end

            composer_containers = composer_container_client.get_by_composer_service_id(composer_service.id)

            cmdline = "docker-compose up -d #{composer_containers.map(&:name).join(' ')}"

            cmd = TTY::Command.new(output: Yadecli.LOGGER)

            puts ''
            print "Executing command '#{cmdline}'... "

            cmd.run(cmdline, chdir: composer_project.install_dir)

            puts 'done'.colorize(:green)
          rescue TTY::Command::ExitError => e
            puts 'failed'.colorize(:red)
            puts e
          end

          'Yade composer service successfully started'
        end
      end
    end
  end
end